/*
 * Copyright (c) 2014-2018, Sonos, Inc.
 *
 * SPDX-License-Identifier: GPL-2.0
 *
 * sonos_attr.c.inc: Sonos attribute common code
 */

#include "sonos_attr.h"

int
sonosIsKnownAttribute(uint32_t id)
{
    id &= ~SONOS_ATTR_CRIT_BIT;

    return 0 < id && id <= SONOS_ATTR_ID_MAX;
}

int
sonosHasDuplicateAttribute(uint32_t numUnsignedAttrs,
                           const SonosAttribute unsignedAttrs[],
                           uint32_t numSignedAttrs,
                           const SonosAttribute signedAttrs[])
{
    uint32_t i, j;
    uint32_t iAttr, jAttr;

    for (i = 0; i < numUnsignedAttrs; i++) {
        iAttr = SONOS_ATTR_ID(unsignedAttrs[i].attributeId);

        for (j = i + 1; j < numUnsignedAttrs; j++) {
            jAttr = SONOS_ATTR_ID(unsignedAttrs[j].attributeId);
            if (iAttr == jAttr) {
                return 1;
            }
        }
        for (j = 0; j < numSignedAttrs; j++) {
            jAttr = SONOS_ATTR_ID(signedAttrs[j].attributeId);
            if (iAttr == jAttr) {
                return 1;
            }
        }
    }

    for (i = 0; i < numSignedAttrs; i++) {
        iAttr = SONOS_ATTR_ID(signedAttrs[i].attributeId);

        for (j = i + 1; j < numSignedAttrs; j++) {
            jAttr = SONOS_ATTR_ID(signedAttrs[j].attributeId);
            if (iAttr == jAttr) {
                return 1;
            }
        }
    }

    return 0;
}

const char *
sonosContentTypeToString(SonosContentType_t ct)
{
    const char* ctStrings[1+SONOS_CT_MAX] =
    {
        "invalid",
        "UPD",
        "Kernel",
        "FPGA",
        "M4",
        "LK",
        "TEE",
        "U-Boot",
        "BL Combined",
        "BL30",
        "BL31",
        "BL32",
        "BL33",
        "FIP",
    };

    return (ct <= SONOS_CT_MAX) ? ctStrings[ct] : "unknown";
}

#ifdef SONOS_ARCH_ATTR_DEBUG_SIGNED_ATTRIBUTES

#include <inttypes.h>
#include <string.h>

static void
sonosAttributeDumpHex(FILE* f, const char* indent,
                      const void* vbuf, size_t bufLen)
{
    size_t i, mod, finalChar;
    const uint8_t *buf = vbuf;

    for (i = 0; i < bufLen; i++) {
        mod = i % 8;
        finalChar = i == bufLen - 1;

        fprintf(f, "%s0x%02x%s%s",
                mod == 0 ? indent : "",
                buf[i],
                finalChar ? "" : ",",
                mod == 7 || finalChar ? "\n" : " ");
    }
}

static const char* ctStringsLower[1+SONOS_CT_MAX] =
{
    "invalid",
    "upd",
    "kernel",
    "fpga",
    "m4",
    "lk",
    "tee",
    "uboot",
    "bl_combined",
    "bl30",
    "bl31",
    "bl32",
    "bl33",
    "fip",
};

SonosContentType_t
sonosContentTypeFromString(const char* s)
{
    size_t i;

    for (i = 1; i < (1+SONOS_CT_MAX); i++) {
        if (strcmp(s, ctStringsLower[i]) == 0) {
            return (SonosContentType_t)i;
        }
    }
    return SONOS_CT_INVALID;
}

void
sonosAttributePrint(FILE* f, const SonosAttribute* a)
{
    uint32_t id = SONOS_ATTR_ID(a->attributeId);
    uint32_t len = a->attributeValueLen;
    const char* attrName = NULL;
    char attrNameBuf[32];
    const char* attrPrettyValue = NULL;
    char attrPrettyValueBuf[64];
    const char* wlTypeStrings[] =
    {
         "cpuid", "serial", "cpuid16"
    };
    const size_t wlEntrySize[] =
    {
         SONOS_FWW_LEN_CPUID,
         SONOS_FWW_LEN_SERIAL,
         SONOS_FWW_LEN_CPUID16
    };

    switch (id) {
    case SONOS_ATTR_ID_PADDING:
        attrName = "padding";
        break;
    case SONOS_ATTR_ID_MESSAGE_DIGEST:
        attrName = "message digest";
        break;
    case SONOS_ATTR_ID_CONTENT_TYPE:
    {
        SonosContentType_t contentType;
        attrName = "content-type";
        attrPrettyValue = attrPrettyValueBuf;
        contentType = a->x.ct.contentType;
        if (contentType <= SONOS_CT_MAX) {
            snprintf(attrPrettyValueBuf, sizeof attrPrettyValueBuf, "%s",
                     ctStringsLower[contentType]);
        }
        else {
            snprintf(attrPrettyValueBuf, sizeof attrPrettyValueBuf,
                     "%s (%"PRIu32")",
                     "unknown", contentType);
        }
        break;
    }
    case SONOS_ATTR_ID_SRK_REVOKE:
    {
        SonosSrkRevoke_t srkValue;
        attrName = "SRK revoke";
        attrPrettyValue = attrPrettyValueBuf;
        srkValue = a->x.sr.srkRevokeFuse;
        snprintf(attrPrettyValueBuf, sizeof attrPrettyValueBuf,
                 "0x%x", (unsigned)srkValue);
        break;
    }
    case SONOS_ATTR_ID_WHITELIST:
        attrName = "whitelist";
        break;
    case SONOS_ATTR_ID_AR_VERSION:
    {
        attrName = "AR version";
        attrPrettyValue = attrPrettyValueBuf;
        snprintf(attrPrettyValueBuf, sizeof attrPrettyValueBuf,
                 "0x%" PRIu32, a->x.arv.arVersion);
        break;
    }
    default:
        attrName = attrNameBuf;
        snprintf(attrNameBuf, sizeof attrNameBuf, "unknown (%"PRIu32")", id);
        break;
    }

    fprintf(f, "%s %s(%"PRIu32" bytes):\n",
            attrName,
            SONOS_ATTR_IS_CRITICAL(a->attributeId) ? "(critical) " : "",
            len);
    if (id != SONOS_ATTR_ID_WHITELIST) {
        if (attrPrettyValue) {
            fprintf(f, "  pretty value: %s\n", attrPrettyValue);
        }
        fprintf(f, "  raw value:\n");
        sonosAttributeDumpHex(f, "    ", a->x.attributeValue, len);
    }
    else {
        SonosAttributeWhitelist wl;
        size_t i, entrySize = 0;
        const uint8_t *buf;

        wl = a->x.wl;
        switch (wl.header.whitelistType) {
            case SONOS_FWW_TYPE_CPUID:
            case SONOS_FWW_TYPE_SERIAL:
            case SONOS_FWW_TYPE_CPUID16:
                fprintf(f, "  pretty value: %s %"PRIu32"\n",
                    wlTypeStrings[wl.header.whitelistType-1],
                    wl.header.numEntries);
                entrySize = wlEntrySize[wl.header.whitelistType-1];
                break;
            default:
                fprintf(f, "unknown type %d\n", wl.header.whitelistType);
                return;
        }
        buf = wl.entriesPtr;
        for (i = 0; i < wl.header.numEntries; ++i) {
            sonosAttributeDumpHex(f, "    ", buf, entrySize);
            buf += entrySize;
        }
    }
}

#endif
